@@ -8,18 +8,25 @@ if TYPE_CHECKING:
     from streamflow.deployment.deployment_manager import ModelConfig
     from streamflow.core.context import StreamFlowContext
     from streamflow.core.deployment import Connector
+    from streamflow.core.scheduling import Hardware
     from typing import Optional, MutableMapping, Any, Set, Union
 
 
 class Command(ABC):
 
     def __init__(self, step: Step):
+        super().__init__()
         self.step: Step = step
 
     @abstractmethod
     async def execute(self, job: Job) -> CommandOutput:
         ...
 
+    async def skip(self, job: Job) -> CommandOutput:
+        return CommandOutput(
+            value=None,
+            status=Status.SKIPPED)
+
 
 class CommandOutput(object):
     __slots__ = ('value', 'status')
@@ -34,6 +41,23 @@ class CommandOutput(object):
         return CommandOutput(value=value, status=self.status)
 
 
+class Condition(ABC):
+
+    def __init__(self, step: Step):
+        self.step: Step = step
+
+    @abstractmethod
+    async def eval(self, job: Job) -> bool:
+        ...
+
+
+class HardwareRequirement(ABC):
+
+    @abstractmethod
+    def eval(self, inputs: MutableSequence[Token]) -> Hardware:
+        ...
+
+
 class Token(object):
     __slots__ = ('name', 'value', 'job', 'tag', 'weight')
 
@@ -41,7 +65,7 @@ class Token(object):
                  name: str,
                  value: Any,
                  job: Optional[Union[str, MutableSequence[str]]] = None,
-                 tag: str = '/',
+                 tag: str = '0',
                  weight: int = 0):
         self.name = name
         self.value: Any = value
@@ -55,6 +79,9 @@ class Token(object):
     def rename(self, name: str) -> Token:
         return Token(name=name, job=self.job, tag=self.tag, weight=self.weight, value=self.value)
 
+    def retag(self, tag: str) -> Token:
+        return Token(name=self.name, job=self.job, tag=tag, weight=self.weight, value=self.value)
+
 
 class TerminationToken(Token):
 
@@ -67,6 +94,9 @@ class TerminationToken(Token):
     def rename(self, name: str) -> Token:
         return TerminationToken(name=name)
 
+    def retag(self, tag: str) -> Token:
+        raise NotImplementedError()
+
 
 class Executor(ABC):
 
@@ -79,7 +109,7 @@ class Executor(ABC):
 
 
 class Job(ABC):
-    __slots__ = ('name', 'step', 'inputs', 'input_directory', 'output_directory', 'tmp_directory')
+    __slots__ = ('name', 'step', 'inputs', 'input_directory', 'output_directory', 'tmp_directory', 'hardware')
 
     def __init__(self,
                  name: str,
@@ -87,13 +117,15 @@ class Job(ABC):
                  inputs: MutableSequence[Token],
                  input_directory: Optional[str] = None,
                  output_directory: Optional[str] = None,
-                 tmp_directory: Optional[str] = None):
+                 tmp_directory: Optional[str] = None,
+                 hardware: Optional[Hardware] = None):
         self.name: str = name
         self.step = step
         self.inputs: MutableSequence[Token] = inputs
         self.input_directory: Optional[str] = input_directory
         self.output_directory: Optional[str] = output_directory
         self.tmp_directory: Optional[str] = tmp_directory
+        self.hardware: Optional[Hardware] = hardware
 
     def get_resources(self, statuses: Optional[MutableSequence[Status]] = None) -> MutableSequence[str]:
         return self.step.context.scheduler.get_resources(self.name, statuses)
@@ -214,18 +246,24 @@ class Step(ABC):
                  name: str,
                  context: StreamFlowContext,
                  command: Optional[Command] = None,
-                 target: Optional[Target] = None):
+                 target: Optional[Target] = None,
+                 workflow: Optional[Workflow] = None):
         super().__init__()
         self.context: StreamFlowContext = context
         self.command: Optional[Command] = command
+        self.condition: Optional[Condition] = None
+        self.hardware_requirement: Optional[HardwareRequirement] = None
         self.input_combinator: Optional[InputCombinator] = None
-        self.input_ports: MutableMapping[str, Union[InputPort, InputCombinator]] = {}
+        self.input_ports: MutableMapping[str, InputPort] = {}
         self.name: str = name
         self.output_ports: MutableMapping[str, OutputPort] = {}
+        self.persistent_id: Optional[int] = None
+        self.scheduling_group: Optional[str] = None
         self.status: Status = Status.WAITING
         self.target: Optional[Target] = target
         self.terminated: bool = False
         self.workdir: Optional[str] = None
+        self.workflow: Optional[Workflow] = workflow
 
     @abstractmethod
     def get_connector(self) -> Optional[Connector]:
@@ -257,4 +295,9 @@ class Workflow(object):
     def __init__(self):
         super().__init__()
         self.steps: MutableMapping[str, Step] = {}
+        self.root_steps: MutableSequence[str] = []
         self.output_ports: MutableMapping[str, OutputPort] = {}
+
+    def add_step(self, step: Step):
+        self.steps[step.name] = step
+        step.workflow = self
