@@ -15,7 +15,6 @@ if TYPE_CHECKING:
     from streamflow.core.context import StreamFlowContext
     from streamflow.core.workflow import Workflow
     from typing import Any, MutableMapping, Optional
-    from typing_extensions import Text
 
 
 class StreamFlowExecutor(Executor):
@@ -26,11 +25,11 @@ class StreamFlowExecutor(Executor):
         super().__init__(workflow)
         self.context: StreamFlowContext = context
         self.executions: MutableSequence[Task] = []
-        self.output_tasks: MutableSequence[Task] = []
-        self.received: MutableSequence[Text] = []
+        self.output_tasks: MutableMapping[str, Task] = {}
+        self.received: MutableSequence[str] = []
         self.closed: bool = False
 
-    async def _execute(self, step: Text):
+    async def _execute(self, step: str):
         try:
             await self.workflow.steps[step].run()
         except Exception as e:
@@ -45,15 +44,17 @@ class StreamFlowExecutor(Executor):
         self.closed = True
 
     async def _wait_outputs(self,
-                            output_consumer: Text,
-                            output_dir: Text,
-                            output_tokens: MutableMapping[Text, Any]) -> MutableMapping[Text, Any]:
-        finished, unfinished = await asyncio.wait(self.output_tasks, return_when=FIRST_COMPLETED)
-        self.output_tasks = list(unfinished)
+                            output_consumer: str,
+                            output_dir: str,
+                            output_tokens: MutableMapping[str, Any]) -> MutableMapping[str, Any]:
+        finished, unfinished = await asyncio.wait(self.output_tasks.values(), return_when=FIRST_COMPLETED)
+        self.output_tasks = {t.get_name(): t for t in unfinished}
         for task in finished:
             if task.cancelled():
                 continue
             task_name = cast(Task, task).get_name()
+            if task_name not in self.workflow.output_ports:
+                continue
             token = task.result()
             # If a TerminationToken is received, the corresponding port terminated its outputs
             if isinstance(token, TerminationToken):
@@ -61,19 +62,24 @@ class StreamFlowExecutor(Executor):
                 # When the last port terminates, the entire executor terminates
                 if len(self.received) == len(self.workflow.output_ports):
                     self.closed = True
-                    return output_tokens
             else:
                 # Collect outputs
                 token_processor = self.workflow.output_ports[task_name].token_processor
                 token = await token_processor.collect_output(token, output_dir)
-                if token.value is not None:
-                    output_tokens[task_name] = utils.get_token_value(token)
-                # Create a new task in place of the completed one
-                self.output_tasks.append(asyncio.create_task(
-                    self.workflow.output_ports[task_name].get(output_consumer), name=task_name))
+                output_tokens[task_name] = utils.get_token_value(token)
+                # Create a new task in place of the completed one if not terminated
+                if task_name not in self.received:
+                    self.output_tasks[task_name] = asyncio.create_task(
+                        self.workflow.output_ports[task_name].get(output_consumer), name=task_name)
+        # Check if new output ports have been created
+        for port_name, port in self.workflow.output_ports.items():
+            if port_name not in self.output_tasks and port_name not in self.received:
+                self.output_tasks[port_name] = asyncio.create_task(port.get(output_consumer), name=port_name)
+                self.closed = False
+        # Return output tokens
         return output_tokens
 
-    async def run(self, output_dir: Optional[Text] = os.getcwd()):
+    async def run(self, output_dir: Optional[str] = os.getcwd()):
         output_tokens = {}
         # Execute workflow
         for step in self.workflow.steps:
@@ -84,7 +90,7 @@ class StreamFlowExecutor(Executor):
             # Retreive output tokens
             output_consumer = utils.random_name()
             for port_name, port in self.workflow.output_ports.items():
-                self.output_tasks.append(asyncio.create_task(port.get(output_consumer), name=port_name))
+                self.output_tasks[port_name] = asyncio.create_task(port.get(output_consumer), name=port_name)
             while not self.closed:
                 output_tokens = await self._wait_outputs(output_consumer, output_dir, output_tokens)
         # Otherwise simply wait for all tasks to finish
